@@ -1,7 +1,7 @@
 /*
  * frogr-details-dialog.c -- Picture details dialog
  *
- * Copyright (C) 2009 Mario Sanchez Prada
+ * Copyright (C) 2009, 2010 Mario Sanchez Prada
  * Authors: Mario Sanchez Prada <msanchez@igalia.com>
  *
  * This program is free software; you can redistribute it and/or
@@ -20,10 +20,14 @@
  *
  */
 
-#include <config.h>
 #include "frogr-details-dialog.h"
+
 #include "frogr-controller.h"
 #include "frogr-picture.h"
+#include "frogr-util.h"
+
+#include <config.h>
+#include <glib/gi18n.h>
 
 #define GTKBUILDER_FILE APP_DATA_DIR "/gtkbuilder/frogr-details-dialog.xml"
 #define MPICTURES_IMAGE APP_DATA_DIR "/images/mpictures.png"
@@ -34,9 +38,9 @@
 #define PICTURE_WIDTH 150
 #define PICTURE_HEIGHT 150
 
-#define FROGR_DETAILS_DIALOG_GET_PRIVATE(object)            \
-  (G_TYPE_INSTANCE_GET_PRIVATE ((object),                   \
-                                FROGR_TYPE_DETAILS_DIALOG,  \
+#define FROGR_DETAILS_DIALOG_GET_PRIVATE(object)                \
+  (G_TYPE_INSTANCE_GET_PRIVATE ((object),                       \
+                                FROGR_TYPE_DETAILS_DIALOG,      \
                                 FrogrDetailsDialogPrivate))
 
 G_DEFINE_TYPE (FrogrDetailsDialog, frogr_details_dialog, GTK_TYPE_DIALOG);
@@ -52,7 +56,7 @@ typedef struct _FrogrDetailsDialogPrivate {
   GtkTextBuffer *text_buffer;
   GtkWidget *picture_img;
   GtkWidget *mpictures_label;
-  GSList *fpictures;
+  GSList *pictures;
 } FrogrDetailsDialogPrivate;
 
 /* Properties */
@@ -64,30 +68,37 @@ enum  {
 
 /* Prototypes */
 
-static void _update_ui (FrogrDetailsDialog *fdetailsdialog);
+static void _update_ui (FrogrDetailsDialog *self);
 static GdkPixbuf *_get_scaled_pixbuf (GdkPixbuf *pixbuf);
-static void _fill_dialog_with_data (FrogrDetailsDialog *fdetailsdialog);
-static gboolean _validate_dialog_data (FrogrDetailsDialog *fdetailsdialog);
-static gboolean _save_data (FrogrDetailsDialog *fdetailsdialog);
+static void _fill_dialog_with_data (FrogrDetailsDialog *self);
+static gboolean _validate_dialog_data (FrogrDetailsDialog *self);
+static gboolean _save_data (FrogrDetailsDialog *self);
 
 void _on_public_private_rbutton_toggled (GtkToggleButton *tbutton,
                                          gpointer data);
 void _on_family_friend_cbutton_toggled (GtkToggleButton *tbutton,
                                         gpointer data);
 
+static void _dialog_response_cb (GtkDialog *dialog, gint response, gpointer data);
+
+
 /* Private API */
 
 static void
-_update_ui (FrogrDetailsDialog *fdetailsdialog)
+_update_ui (FrogrDetailsDialog *self)
 {
   FrogrDetailsDialogPrivate *priv =
-    FROGR_DETAILS_DIALOG_GET_PRIVATE (fdetailsdialog);
+    FROGR_DETAILS_DIALOG_GET_PRIVATE (self);
   gboolean active;
 
   /* Adjust sensitiveness for check buttons */
   active = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (priv->public_rb));
   gtk_widget_set_sensitive (priv->friend_cb, !active);
   gtk_widget_set_sensitive (priv->family_cb, !active);
+
+  /* Initial widget to grab focus */
+  gtk_widget_grab_focus (priv->title_entry);
+  gtk_editable_set_position (GTK_EDITABLE (priv->title_entry), -1);
 }
 
 static GdkPixbuf *
@@ -122,12 +133,12 @@ _get_scaled_pixbuf (GdkPixbuf *pixbuf)
 }
 
 static void
-_fill_dialog_with_data (FrogrDetailsDialog *fdetailsdialog)
+_fill_dialog_with_data (FrogrDetailsDialog *self)
 {
   FrogrDetailsDialogPrivate *priv =
-    FROGR_DETAILS_DIALOG_GET_PRIVATE (fdetailsdialog);
+    FROGR_DETAILS_DIALOG_GET_PRIVATE (self);
 
-  FrogrPicture *fpicture;
+  FrogrPicture *picture;
   GSList *item;
   guint n_pictures;
   gchar *title_val = NULL;
@@ -138,15 +149,15 @@ _fill_dialog_with_data (FrogrDetailsDialog *fdetailsdialog)
   gboolean is_family_val = FALSE;
 
   /* Take first element values */
-  item = priv->fpictures;
-  fpicture = FROGR_PICTURE (item->data);
+  item = priv->pictures;
+  picture = FROGR_PICTURE (item->data);
 
-  title_val = (gchar *)frogr_picture_get_title (fpicture);
-  desc_val = (gchar *)frogr_picture_get_description (fpicture);
-  tags_val = (gchar *)frogr_picture_get_tags (fpicture);
-  is_public_val = frogr_picture_is_public (fpicture);
-  is_friend_val = frogr_picture_is_friend (fpicture);
-  is_family_val = frogr_picture_is_family (fpicture);
+  title_val = (gchar *)frogr_picture_get_title (picture);
+  desc_val = (gchar *)frogr_picture_get_description (picture);
+  tags_val = (gchar *)frogr_picture_get_tags (picture);
+  is_public_val = frogr_picture_is_public (picture);
+  is_friend_val = frogr_picture_is_friend (picture);
+  is_family_val = frogr_picture_is_family (picture);
 
   /* Iterate over the rest of elements */
   for (item = g_slist_next (item); item; item = g_slist_next (item))
@@ -159,20 +170,20 @@ _fill_dialog_with_data (FrogrDetailsDialog *fdetailsdialog)
       gboolean is_family = FALSE;
 
       /* Retrieve needed data */
-      fpicture = FROGR_PICTURE (item->data);
+      picture = FROGR_PICTURE (item->data);
 
       /* Only retrieve title and desc when needed */
       if (title_val != NULL)
-        title = (gchar *)frogr_picture_get_title (fpicture);
+        title = (gchar *)frogr_picture_get_title (picture);
       if (desc_val != NULL)
-        desc = (gchar *)frogr_picture_get_description (fpicture);
+        desc = (gchar *)frogr_picture_get_description (picture);
       if (tags_val != NULL)
-        tags = (gchar *)frogr_picture_get_tags (fpicture);
+        tags = (gchar *)frogr_picture_get_tags (picture);
 
       /* boolean properties */
-      is_public = frogr_picture_is_public (fpicture);
-      is_friend = frogr_picture_is_friend (fpicture);
-      is_family = frogr_picture_is_family (fpicture);
+      is_public = frogr_picture_is_public (picture);
+      is_friend = frogr_picture_is_friend (picture);
+      is_family = frogr_picture_is_family (picture);
 
       /* Update actual values for the dialog */
       if (title_val && title)
@@ -195,8 +206,6 @@ _fill_dialog_with_data (FrogrDetailsDialog *fdetailsdialog)
         {
           gtk_toggle_button_set_inconsistent (GTK_TOGGLE_BUTTON (priv->public_rb),
                                               is_public_val != is_public);
-          gtk_toggle_button_set_inconsistent (GTK_TOGGLE_BUTTON (priv->public_rb),
-                                              is_public_val != is_public);
         }
       if (!gtk_toggle_button_get_inconsistent (GTK_TOGGLE_BUTTON (priv->family_cb)))
         {
@@ -234,7 +243,7 @@ _fill_dialog_with_data (FrogrDetailsDialog *fdetailsdialog)
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->family_cb),
                                 is_family_val);
 
-  n_pictures = g_slist_length (priv->fpictures);
+  n_pictures = g_slist_length (priv->pictures);
   if (n_pictures > 1)
     {
       GdkPixbuf *pixbuf;
@@ -247,14 +256,14 @@ _fill_dialog_with_data (FrogrDetailsDialog *fdetailsdialog)
 
       /* Show the hidden label */
       gtk_widget_show (priv->mpictures_label);
-      text = g_strdup_printf ("(Editing %d pictures)", n_pictures);
+      text = g_strdup_printf (_("(Editing %d pictures)"), n_pictures);
       gtk_label_set_text (GTK_LABEL (priv->mpictures_label), text);
       g_free (text);
     }
   else
     {
       /* Set pixbuf scaled to the right size */
-      GdkPixbuf *pixbuf = frogr_picture_get_pixbuf (fpicture);
+      GdkPixbuf *pixbuf = frogr_picture_get_pixbuf (picture);
       GdkPixbuf *s_pixbuf = _get_scaled_pixbuf (pixbuf);
       gtk_image_set_from_pixbuf (GTK_IMAGE (priv->picture_img), s_pixbuf);
       g_object_unref (s_pixbuf);
@@ -264,19 +273,19 @@ _fill_dialog_with_data (FrogrDetailsDialog *fdetailsdialog)
     }
 
   /* Update UI */
-  _update_ui (fdetailsdialog);
+  _update_ui (self);
 }
 
 static gboolean
-_validate_dialog_data (FrogrDetailsDialog *fdetailsdialog)
+_validate_dialog_data (FrogrDetailsDialog *self)
 {
   FrogrDetailsDialogPrivate *priv =
-    FROGR_DETAILS_DIALOG_GET_PRIVATE (fdetailsdialog);
+    FROGR_DETAILS_DIALOG_GET_PRIVATE (self);
 
   gboolean result = TRUE;
 
   /* Mandatory fields (only if editing a single picture) */
-  if (g_slist_length (priv->fpictures) <= 1)
+  if (g_slist_length (priv->pictures) <= 1)
     {
       gchar *title =
         g_strdup (gtk_entry_get_text (GTK_ENTRY (priv->title_entry)));
@@ -292,10 +301,10 @@ _validate_dialog_data (FrogrDetailsDialog *fdetailsdialog)
 }
 
 static gboolean
-_save_data (FrogrDetailsDialog *fdetailsdialog)
+_save_data (FrogrDetailsDialog *self)
 {
   FrogrDetailsDialogPrivate *priv =
-    FROGR_DETAILS_DIALOG_GET_PRIVATE (fdetailsdialog);
+    FROGR_DETAILS_DIALOG_GET_PRIVATE (self);
 
   GtkTextIter start;
   GtkTextIter end;
@@ -338,28 +347,28 @@ _save_data (FrogrDetailsDialog *fdetailsdialog)
     }
 
   /* validate dialog */
-  if (_validate_dialog_data (fdetailsdialog))
+  if (_validate_dialog_data (self))
     {
-      FrogrPicture *fpicture;
+      FrogrPicture *picture;
       GSList *item;
       guint n_pictures;
 
       /* Iterate over the rest of elements */
-      n_pictures = g_slist_length (priv->fpictures);
-      for (item = priv->fpictures; item; item = g_slist_next (item))
+      n_pictures = g_slist_length (priv->pictures);
+      for (item = priv->pictures; item; item = g_slist_next (item))
         {
-          fpicture = FROGR_PICTURE (item->data);
+          picture = FROGR_PICTURE (item->data);
 
           if (!g_str_equal (title, "") || (n_pictures <= 1))
-            frogr_picture_set_title (fpicture, title);
+            frogr_picture_set_title (picture, title);
           if (!g_str_equal (description, "") || (n_pictures <= 1))
-            frogr_picture_set_description (fpicture, description);
+            frogr_picture_set_description (picture, description);
           if (!g_str_equal (tags, "") || (n_pictures <= 1))
-            frogr_picture_set_tags (fpicture, tags);
+            frogr_picture_set_tags (picture, tags);
 
-          frogr_picture_set_public (fpicture, is_public);
-          frogr_picture_set_friend (fpicture, is_friend);
-          frogr_picture_set_family (fpicture, is_family);
+          frogr_picture_set_public (picture, is_public);
+          frogr_picture_set_friend (picture, is_friend);
+          frogr_picture_set_family (picture, is_family);
 
           /* Everything went fine */
           result = TRUE;
@@ -367,19 +376,8 @@ _save_data (FrogrDetailsDialog *fdetailsdialog)
     }
   else
     {
-      /* Show alert */
-      GtkWindow *parent_win =
-        gtk_window_get_transient_for (GTK_WINDOW (fdetailsdialog));
-
-      GtkWidget *dialog =
-        gtk_message_dialog_new (parent_win,
-                                GTK_DIALOG_MODAL,
-                                GTK_MESSAGE_WARNING,
-                                GTK_BUTTONS_CLOSE,
-                                "Missing data required");
-      /* Run alert dialog */
-      gtk_dialog_run (GTK_DIALOG (dialog));
-      gtk_widget_destroy (dialog);
+      /* This shows a dialog notifying the problem to the user */
+      frogr_util_show_error_dialog (GTK_WINDOW (self), _("Missing data required"));
     }
 
   /* free */
@@ -397,7 +395,7 @@ void
 _on_public_private_rbutton_toggled (GtkToggleButton *tbutton,
                                     gpointer data)
 {
-  FrogrDetailsDialog *fdetailsdialog = FROGR_DETAILS_DIALOG (data);
+  FrogrDetailsDialog *self = FROGR_DETAILS_DIALOG (data);
   FrogrDetailsDialogPrivate *priv = FROGR_DETAILS_DIALOG_GET_PRIVATE (data);
 
   /* Reset consistence and update UI */
@@ -405,18 +403,29 @@ _on_public_private_rbutton_toggled (GtkToggleButton *tbutton,
                                       FALSE);
   gtk_toggle_button_set_inconsistent (GTK_TOGGLE_BUTTON (priv->private_rb),
                                       FALSE);
-  _update_ui (fdetailsdialog);
+  _update_ui (self);
 }
 
 void
 _on_family_friend_cbutton_toggled (GtkToggleButton *tbutton,
                                    gpointer data)
 {
-  FrogrDetailsDialog *fdetailsdialog = FROGR_DETAILS_DIALOG (data);
+  FrogrDetailsDialog *self = FROGR_DETAILS_DIALOG (data);
 
   /* Reset consistence and update UI */
   gtk_toggle_button_set_inconsistent (tbutton, FALSE);
-  _update_ui (fdetailsdialog);
+  _update_ui (self);
+}
+
+static void _dialog_response_cb (GtkDialog *dialog, gint response, gpointer data)
+{
+  FrogrDetailsDialog *self = FROGR_DETAILS_DIALOG (dialog);
+
+  /* Try to save data if response is OK */
+  if (response == GTK_RESPONSE_OK && _save_data (self) == FALSE)
+      return;
+
+  gtk_widget_destroy (GTK_WIDGET (self));
 }
 
 static void
@@ -430,7 +439,7 @@ _frogr_details_dialog_set_property (GObject *object,
   switch (prop_id)
     {
     case PROP_PICTURES:
-      priv->fpictures = (GSList *) g_value_get_pointer (value);
+      priv->pictures = (GSList *) g_value_get_pointer (value);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -449,7 +458,7 @@ _frogr_details_dialog_get_property (GObject *object,
   switch (prop_id)
     {
     case PROP_PICTURES:
-      g_value_set_pointer (value, priv->fpictures);
+      g_value_set_pointer (value, priv->pictures);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -458,12 +467,18 @@ _frogr_details_dialog_get_property (GObject *object,
 }
 
 static void
-_frogr_details_dialog_finalize (GObject *object)
+_frogr_details_dialog_dispose (GObject *object)
 {
   FrogrDetailsDialogPrivate *priv = FROGR_DETAILS_DIALOG_GET_PRIVATE (object);
-  g_slist_foreach (priv->fpictures, (GFunc)g_object_unref, NULL);
-  g_slist_free (priv->fpictures);
-  G_OBJECT_CLASS(frogr_details_dialog_parent_class)->finalize (object);
+
+  if (priv->pictures)
+    {
+      g_slist_foreach (priv->pictures, (GFunc)g_object_unref, NULL);
+      g_slist_free (priv->pictures);
+      priv->pictures = NULL;
+    }
+
+  G_OBJECT_CLASS(frogr_details_dialog_parent_class)->dispose (object);
 }
 
 static void
@@ -472,10 +487,10 @@ frogr_details_dialog_class_init (FrogrDetailsDialogClass *klass)
   GObjectClass *obj_class = (GObjectClass *)klass;
   GParamSpec *pspec;
 
-  /* GtkObject signals */
+  /* GObject signals */
   obj_class->set_property = _frogr_details_dialog_set_property;
   obj_class->get_property = _frogr_details_dialog_get_property;
-  obj_class->finalize = _frogr_details_dialog_finalize;
+  obj_class->dispose = _frogr_details_dialog_dispose;
 
   /* Install properties */
   pspec = g_param_spec_pointer ("pictures",
@@ -490,10 +505,10 @@ frogr_details_dialog_class_init (FrogrDetailsDialogClass *klass)
 }
 
 static void
-frogr_details_dialog_init (FrogrDetailsDialog *fdetailsdialog)
+frogr_details_dialog_init (FrogrDetailsDialog *self)
 {
   FrogrDetailsDialogPrivate *priv =
-    FROGR_DETAILS_DIALOG_GET_PRIVATE (fdetailsdialog);
+    FROGR_DETAILS_DIALOG_GET_PRIVATE (self);
 
   GtkBuilder *builder;
   GtkWidget *main_vbox;
@@ -503,10 +518,15 @@ frogr_details_dialog_init (FrogrDetailsDialog *fdetailsdialog)
   gtk_builder_add_from_file (builder, GTKBUILDER_FILE, NULL);
 
   main_vbox = GTK_WIDGET (gtk_builder_get_object (builder, "main-vbox"));
+#if GTK_CHECK_VERSION (2,14,0)
   gtk_widget_reparent (main_vbox,
-                       GTK_WIDGET (GTK_DIALOG (fdetailsdialog)->vbox));
+                       GTK_WIDGET (gtk_dialog_get_content_area (GTK_DIALOG (self))));
+#else
+  gtk_widget_reparent (main_vbox,
+                       GTK_WIDGET (GTK_DIALOG (self)->vbox));
+#endif
 
-  gtk_dialog_add_buttons (GTK_DIALOG (fdetailsdialog),
+  gtk_dialog_add_buttons (GTK_DIALOG (self),
                           GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                           GTK_STOCK_OK, GTK_RESPONSE_OK,
                           NULL);
@@ -534,11 +554,12 @@ frogr_details_dialog_init (FrogrDetailsDialog *fdetailsdialog)
     GTK_WIDGET (gtk_builder_get_object (builder, "mpictures_label"));
 
   /* Connect signals */
-  gtk_builder_connect_signals (builder, fdetailsdialog);
+  gtk_builder_connect_signals (builder, self);
 
   /* Show the UI */
-  gtk_widget_show_all (GTK_WIDGET (fdetailsdialog));
-
+  gtk_widget_show_all (GTK_WIDGET (self));
+  gtk_dialog_set_default_response (GTK_DIALOG (self),
+                                   GTK_RESPONSE_OK);
   /* Free */
   g_object_unref (G_OBJECT (builder));
 }
@@ -546,41 +567,26 @@ frogr_details_dialog_init (FrogrDetailsDialog *fdetailsdialog)
 
 /* Public API */
 
-FrogrDetailsDialog *
-frogr_details_dialog_new (GtkWindow *parent, GSList *fpictures)
-{
-  GObject *new = g_object_new (FROGR_TYPE_DETAILS_DIALOG,
-                               "modal", TRUE,
-                               "pictures", fpictures,
-                               "transient-for", parent,
-                               "width-request", DIALOG_MIN_WIDTH,
-                               "height-request", DIALOG_MIN_HEIGHT,
-                               "resizable", TRUE,
-                               "title", "Edit picture details",
-                               NULL);
-  return FROGR_DETAILS_DIALOG (new);
-}
-
 void
-frogr_details_dialog_show (FrogrDetailsDialog *fdetailsdialog)
+frogr_details_dialog_show (GtkWindow *parent, GSList *fpictures)
 {
-  gint response;
-  gboolean saved;
+  GtkWidget *dialog = NULL;
+
+  dialog = GTK_WIDGET (g_object_new (FROGR_TYPE_DETAILS_DIALOG,
+                                     "modal", TRUE,
+                                     "pictures", fpictures,
+                                     "transient-for", parent,
+                                     "width-request", DIALOG_MIN_WIDTH,
+                                     "height-request", DIALOG_MIN_HEIGHT,
+                                     "resizable", TRUE,
+                                     "title", _("Edit picture details"),
+                                     NULL));
+
+  g_signal_connect (G_OBJECT (dialog), "response",
+                    G_CALLBACK (_dialog_response_cb), NULL);
 
   /* Fill dialog's widgets with data */
-  _fill_dialog_with_data (fdetailsdialog);
+  _fill_dialog_with_data (FROGR_DETAILS_DIALOG (dialog));
 
-  /* Run the dialog */
-  do
-    {
-      response = gtk_dialog_run (GTK_DIALOG (fdetailsdialog));
-
-      /* Try to save data if response is OK */
-      if (response == GTK_RESPONSE_OK)
-        saved =_save_data (fdetailsdialog);
-
-    } while ((response == GTK_RESPONSE_OK) && !saved);
-
-  /* Destroy the dialog */
-  gtk_widget_destroy (GTK_WIDGET (fdetailsdialog));
+  gtk_widget_show_all (dialog);
 }
